/*
  PV diversion controller with watchdog

  14/01/2019 D. Festing

  SMPS output voltage based on Vmp. 
  
  PD2 Red LED drive
  PD3 70Volts
  PD5
  PD6 Load switch signal
  PD7 61Volts
  PB0 43Volts
  PB1 33Volts
   
  Xtal is 3686400 in makefile

  Fuse settings:
//  hFuse DFh, WDTON on, no BOD
//  hFuse DDh, WDTON off, BODLEVEL 2.7V
//  hfuse CDh, WDTON on, BODLEVEL 2.7v
//  hfuse CCh, WDTON on, BODLEVEL 4.3V
//  SUT1 = 1 and SUT0 = 0 BOD disabled and fast rising
//  SUT1 = 1 and SUT0 = 1 BOD disabled and slow rising
 
  hFuse CCh, WDTON on, BODLEVEL 4.3V 
  lFuse D7h, divide by 8, full swing crystal, SUT1 = 0, SUT0 = 1 when BOD enabled
  efuse FBh, 512 words bootsize, 0x03 was on the Window program??
*/

/*
ToDo:
- maybe leave the ADC running all the time
  
Notes:
- when using interrupts reading/writing variables larger than uint8_t
  need to have interrupts disabled or ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
- disabled digital buffer on ADC4
- ADC is set up for internal reference, ie 5V
*/


#include "project.h" // first file to be included
#include "pv_diversion_controller_io.h"
#include "process_measurements.h"


#define POWER_LEVEL_1 do{PORTD &= ~((1 << 3) | (1 << 7)); PORTB &= ~(1 << 0); PORTB |= (1 << 1);}while(0)
#define POWER_LEVEL_2 do{PORTD &= ~((1 << 3) | (1 << 7)); PORTB &= ~(1 << 1); PORTB |= (1 << 0);}while(0)
#define POWER_LEVEL_3 do{PORTD &= ~(1 << 3); PORTB &= ~((1 << 0) | (1 << 1)); PORTD |= (1 << 7);}while(0)
#define POWER_LEVEL_4 do{PORTD &= ~(1 << 7); PORTB &= ~((1 << 0) | (1 << 1)); PORTD |= (1 << 3);}while(0)
#define POWER_LEVEL_5 do{PORTD &= ~((1 << 3) | (1 << 7)); PORTB &= ~((1 << 0) | (1 << 1));}while(0)


#define POWER1 1
#define POWER2 2
#define POWER3 3
#define POWER4 4
#define POWER5 5


int main(void)
{
   uint16_t averaged_ADC_value = 0;
   uint16_t PV_voltage = 0;
   uint8_t power_state = 0;

// PB0, PB1, PD2, PD3, PD6 and PD7 start off low
   initialise_hardware();

   GREEN_LED_OFF();

// starting off at POWER_LEVEL_1, ie 33V
   POWER_LEVEL_1;
   power_state = POWER1;

   ADC_SELECT_CHANNEL4;
   ADC_ON;
    
// set the watchdog prescalar
   wdt_enable(WDTO_2S); // 2 seconds

// reset the watchdog
   wdt_reset(); 

//   sei();

   FOREVER
   {
   // heartbeat, gives a 1 second delay
      _delay_ms(750);
      GREEN_LED_ON(); 
      _delay_ms(50);
      GREEN_LED_OFF();
      _delay_ms(150);
      GREEN_LED_ON();
      _delay_ms(50); 
      GREEN_LED_OFF();

   // reset the watchdog
      wdt_reset(); 

   // read the PV voltage
      ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
      {
         averaged_ADC_value = get_ADC_value();
      }
       
   /*
      max top multiplier value to fit into an INT = 65535/1024 = 63
      With a 4K7 + 1K2 off the PV panel supply to the ADC input
      and a 1K to ground, 33.0V = 5.0V = 1024 bits. 
  
      Multiply by 10 to shift digits left one place.  Also multiply
      trap value by 10
   */
      PV_voltage = (uint16_t)((averaged_ADC_value * 55) / 170);

      if (PV_voltage > 330)
      {
         PV_voltage = 330; // traps any value over 33.0Volts
      }
      else
         {;}


   // enable load when PV voltage is high enough
      if (PV_voltage >= 290) // 29.0V
      {
         PORTD |= (1 << 6);
      }
      else if (PV_voltage <= 280) // 28.0V
      {
         PORTD &= ~(1 << 6);
      }
      else
         {;}
            

   // adjust Vout
      if (PV_voltage >= 310) // 31.0V
      {
         if (power_state == POWER1)
         {
            POWER_LEVEL_2;

            power_state = POWER2;
         }
         else if (power_state == POWER2)
         {
            POWER_LEVEL_3;

            power_state = POWER3;
         }             
         else if (power_state == POWER3)
         {
            POWER_LEVEL_4;

            power_state = POWER4;
         }
         else if (power_state == POWER4)
         {
            POWER_LEVEL_5;

            power_state = POWER5;
         }
         else
            {;}
      }
      else if (PV_voltage <= 300) // 30.0V
      {
         if (power_state == POWER5)
         {
            POWER_LEVEL_4;

            power_state = POWER4;
         }
         else if (power_state == POWER4)
         {
            POWER_LEVEL_3;

            power_state = POWER3;
         }             
         else if (power_state == POWER3)
         {
            POWER_LEVEL_2;

            power_state = POWER2;
         }
         else if (power_state == POWER2)
         {
            POWER_LEVEL_1;

            power_state = POWER1;
         }
         else
            {;}      
      }
      else
         {;}
 
   // reset the watchdog
      wdt_reset();

   }

   return 0;
}
