// ******************************************************************
//
// File Name   : pv_diversion_controller_io.c
// Author      : D. Festing
// Created     : 23/10/2018
// Revised     : 
// Version     : 1.0
// Target MCU  : AVR ATmega88
// Editor Tabs	: 3
//
// Description : This file contains all initialisation
//               for the AVR IO ports and peripherals 
//
// Notes       : "Lazy refresh" of IO ports
//               void RED_LED_ON(void)
//               {
//                  RED_LED_DDR  |= (1<<RED_LED_DATA_BIT);
//                  RED_LED_PORT |= (1<<RED_LED_DATA_BIT);
//               }
//
//               always check for correct port transition
//               sequence, if used      
//
// ******************************************************************


#include "project.h" // has to be the first header file


// ******************************************************************
//
// Function Name:     initialise_hardware
// Description:       initialises the hardware to the state required 
//                    for this project 
// Preconditions:     None
// Input Parameters:  None
// Return Parameter:  None
// Effects:           
//
// Notes:             This function should be done with interrupts
//                    disabled.
//
// ******************************************************************
void initialise_hardware(void)
{
   uint8_t prescale = 0;
   uint8_t div = 0;

  
/*
Note: If the Watchdog is accidentally enabled, for example by a runaway pointer or brown-out
condition, the device will be reset and the Watchdog Timer will stay enabled. If the code is not
set up to handle the Watchdog, this might lead to an eternal loop of time-out resets. To avoid this
situation, the application software should always clear the Watchdog System Reset Flag
(WDRF) and the WDE control bit in the initialisation routine, even if the Watchdog is not in use.
*/

   MCUSR &= ~(1<<WDRF);
   WDTCSR &= ~(1<<WDE);
   
// On all ports without pull-up or pull-down resistors, either set the port to
// input pull-up enabled or output low, don't leave them floating!


// Port initialisation
// Inputs - Tri-state DDxn = 0 and PORTxn = 0
// Inputs - Pull-up   DDxn = 0 and PORTxn = 1
// Outputs - DDxn = 1 and PORTxn = 0

// PORTB
// PB0 and PB1 output low
// all others input pull-up enabled
   DDRB = 0x03;  // 0000 0011
   PORTB = 0xFC; // 1111 1100
   
// PortC
// PC0, PC1, PC2, PC3 and PC5 input pull-up enabled
// PC4 input pull-up disabled for ADCs
// PC6 reset, input pull-up enabled
// PC7 output low
   DDRC = 0x80;  // 1000 0000
   PORTC = 0x7F; // 0111 1111

// PortD
// PD0, PD1 and PD4 input pull-up enabled
// PD2, PD3, PD5, PD6 and PD7 output low
   DDRD = 0xEC;  // 1110 1100
   PORTD = 0x13; // 0001 0011

	
// ADC initialisation, finds closest division ratio to give < 200kHz
   ADMUX |= (1 << REFS0); // AVCC with external capacitor at AREF pin
//   ADMUX &= ~(1 << REFS0);
//   ADMUX &= ~(1 << REFS1);
// Initialised to external reference
 
  for(div = (F_CPU / 200000L) -1; div > 0; div >>= 1, prescale++)
      {;}
   
   ADCSRA |= prescale;
	
// disable digital input buffers on ADC inputs
   PRR &= ~(1 << PRADC); // ensure PRADC is cleared
   DIDR0 |= (1 << ADC4D);
}


/* ---------- end of hardware_initialise() ---------- */

// ******************************************************************
//
// Function Names:    see below
// Description:       initialises the output ports to the state required
//                    every time the function is called, ie "lazy refresh" 
// Preconditions:     None
// Input Parameters:  None
// Return Parameter:  None
// Effects:           None
//
// Notes:             
//
// ******************************************************************

void RED_LED_ON(void)
{
   RED_LED_DDR  |= (1<<RED_LED_DATA_BIT);
   RED_LED_PORT &= ~(1<<RED_LED_DATA_BIT);
}

void RED_LED_OFF(void)
{
   RED_LED_DDR  |= (1<<RED_LED_DATA_BIT);
   RED_LED_PORT |= (1<<RED_LED_DATA_BIT);  
}

void RED_LED_TOGGLE(void)
{
   RED_LED_DDR  |= (1<<RED_LED_DATA_BIT);
   RED_LED_PORT ^= (1<<RED_LED_DATA_BIT);
}

void GREEN_LED_ON(void)
{
   GREEN_LED_DDR  |= (1<<GREEN_LED_DATA_BIT); 
   GREEN_LED_PORT &= ~(1<<GREEN_LED_DATA_BIT);
}

void GREEN_LED_OFF(void)
{
   GREEN_LED_DDR  |= (1<<GREEN_LED_DATA_BIT);
   GREEN_LED_PORT |= (1<<GREEN_LED_DATA_BIT);
}

void GREEN_LED_TOGGLE(void)
{
   GREEN_LED_DDR  |= (1<<GREEN_LED_DATA_BIT);
   GREEN_LED_PORT ^= (1<<GREEN_LED_DATA_BIT);
}
